#include <R.h>
#include <Rdefines.h>
#include <R_ext/Error.h>

#include <R_ext/eventloop.h>
// #include <R_ext/Rdynload.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <gtkdatabox.h>
#include <gtkdatabox_lines.h>
#include <gtkdatabox_cross_simple.h>

/*----------------------------------------------------------------------- */
static Rboolean gtkOk = 0;

/*----------------------------------------------------------------------- */
// GTK event loop copied from the RGtk2 pacakge by Michael Lawrence (src/Rgtk.c)
void eventHandlerGTK(void *userData) {
    while(gtk_events_pending()) gtk_main_iteration();
}

void R_startGtk(long *rargc, char **rargv, Rboolean *success)
{
    int argc = (int) *rargc;
    if (!gdk_display_get_default()) {
        gtk_disable_setlocale();
        if (!gtk_init_check(&argc, &rargv)) {
            *success = gtkOk = FALSE;
            return;
        }
    }
    InputHandler *h;
    if (!GDK_DISPLAY()) {
        *success = gtkOk = FALSE;
        return;
    }
    h = addInputHandler(R_InputHandlers, ConnectionNumber(GDK_DISPLAY()),
        eventHandlerGTK, -1);
    *success = gtkOk = TRUE;
}

/*----------------------------------------------------------------------- */
static gint onZoomed(GtkDatabox *box);
static gint onSelectionStarted(GtkDatabox *box, GtkDataboxValueRectangle * selectionValues);
static gint onSelectionChanged(GtkDatabox *box, GtkDataboxValueRectangle * selectionValues);
static gint onSelectionCancelled(GtkDatabox *box);
/*----------------------------------------------------------------------- */
SEXP R_dataplot(SEXP x, SEXP y, SEXP z) {
    // R_PreserveObject(x);
    GdkColor color;
    GtkWidget *window, *mbox, *dbox, *table;
    gfloat *xx, *yy;

    int i, nx = LENGTH(x);
    if (nx<1 || nx!=LENGTH(y))
        error("x and y must be of equal length");
    double *sx = REAL(x), *sy = REAL(y);


    /* GTK main window, title, main object container */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_size_request(window, 500, 500);
    g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(gtk_main_quit), NULL);
    gtk_window_set_title(GTK_WINDOW (window), "R: dataplot");
    gtk_container_set_border_width(GTK_CONTAINER(window), 0);
    mbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(window), mbox);

    /* GTK data_box for plotting, background color */
    gtk_databox_create_box_with_scrollbars_and_rulers(&dbox, &table, TRUE, TRUE, TRUE, TRUE);
    color.red = color.green = color.blue = 65535;
    gtk_widget_modify_bg(dbox, GTK_STATE_NORMAL, &color);
    gtk_box_pack_start(GTK_BOX(mbox), table, TRUE, TRUE, 10);

    /* add x,y plot (lines plot for example) */
    xx = g_new0(gfloat, nx);
    /* allocate memory for x and y, copy values from R 
       (R object may seize its existance) on exit */
    yy = g_new0(gfloat, nx);
    for (i=0; i<nx; i++) {
        xx[i] = sx[i];
        yy[i] = sy[i];
    }
    color.red = 65535; color.green = color.blue = 0;
    gtk_databox_graph_add(GTK_DATABOX(dbox), 
        gtk_databox_lines_new(nx, xx, yy, &color, 1));

    /* add x,z plot if z non NULL, potentially one can add as many z's as required */
    if (z!=R_NilValue) {
        if (nx<1 || nx!=LENGTH(z))
            error("x and z must be of equal length");
        yy = g_new0(gfloat, nx);
        sy = REAL(z);
        /* copy z values */
        for (i=0; i<nx; i++)
            yy[i] = sy[i];
        color.red = 0; color.green = color.blue = 65535;
        gtk_databox_graph_add(GTK_DATABOX(dbox), 
            gtk_databox_lines_new(nx, xx, yy, &color, 1));
    }

    /* add axes cross at origin */
    color.red = color.green =color.blue = 0;
    gtk_databox_graph_add(GTK_DATABOX(dbox), 
        gtk_databox_cross_simple_new (&color, 0));

    /* set plot to autorescale */
    gtk_databox_auto_rescale(GTK_DATABOX(dbox), 0.05);

    /* add user-specific callbacks if necessary, gtkdatabox handles zoom automatically */
    /* THESE ARE NOT REQUIRED FOR ZOMMING */

    // g_signal_connect (G_OBJECT (dbox), "databox_zoomed",
    //     G_CALLBACK(onZoomed), NULL);
    // g_signal_connect (G_OBJECT (dbox), "databox_selection_started",
    //     G_CALLBACK (onSelectionStarted), NULL);
    g_signal_connect (G_OBJECT (dbox), "databox_selection_changed",
        G_CALLBACK (onSelectionChanged), NULL);
    // g_signal_connect (G_OBJECT (dbox), "databox_selection_cancelled",
    //     G_CALLBACK (onSelectionCancelled), NULL);
    
    gtk_widget_show_all(window);
    gdk_flush();
    return R_NilValue;
}

/*----------------------------------------------------------------------- */
const gchar* getSignalName(gpointer instance) {
   return g_signal_name(g_signal_get_invocation_hint(instance)->signal_id);
}

static gint onZoomed(GtkDatabox *box)
{
    Rprintf("Processing %s\n", getSignalName(box));
    return 0;
}

static gint onSelectionStarted(GtkDatabox *box, GtkDataboxValueRectangle * selectionValues)
{
    Rprintf("Processing %s\n", getSignalName(box));
    return 0;
}

static gint onSelectionChanged(GtkDatabox *box, GtkDataboxValueRectangle * selectionValues)
{
    // Rprintf("Processing %s\n", getSignalName(box));
    Rprintf(">");
    return 0;
}

static gint onSelectionCancelled(GtkDatabox *box)
{
    Rprintf("Processing %s\n", getSignalName(box));
    return 0;
}
